/*
	bone_ppp.h
	
	template for bone ppp module

	Copyright 2000, Be Incorporated, All Rights Reserved.
*/

#ifndef H_BONE_PPP
#define H_BONE_PPP

#include <module.h>
#include <bone_util.h>

// ***
// PPP Constants
// ***

// PPP Layers
typedef enum
{
	PPPL_TRANSPORT = 	0,
	PPPL_LCP = 			1,
	PPPL_AUTH = 		2,
	PPPL_NETWORK = 		3,
} ppp_level;

// Callbacks used by session to notify transport layer and for sending data
typedef struct ppp_transport_hooks
{
	void *cookie;
	
	status_t	(*send_data)(void *you, bone_data_t *data);
	void		(*open)(void *cookie);
	void		(*close)(void *cookie);
	void 		(*layer)(void *you, ppp_level level);
} ppp_transport_hooks_t;

// 'C pseudo-object class' for handling ppp data of specified type
typedef struct bone_ppp_handler {
	uint16		config_proto;
	uint16		*data_protos;
	ppp_level	level;
	
	void		*cookie;
	
	status_t	(*receive_data)(struct bone_ppp_handler *you, struct bone_data *data, uint16 ppp_proto);
	void		(*lower_layer_up)(struct bone_ppp_handler *you);			
	void		(*lower_layer_down)(struct bone_ppp_handler *you);			
	void		(*open)(struct bone_ppp_handler *you);
	void		(*close)(struct bone_ppp_handler *you);
	
	// Filled out by register_handler()
	struct bone_ppp_session		*session;
} bone_ppp_handler_t;

typedef struct bone_ppp_session_info {

	struct module_info module;
	
	/*
	 * Create/Delete PPP session.
	 */
	status_t	(*init_session)(struct ppp_transport_hooks *hooks, struct bone_ppp_session **session);
	void		(*uninit_session)(struct bone_ppp_session *session);
	
	/*
	 * Open/Close PPP session.
	 */
	status_t	(*open_session)(struct bone_ppp_session *session);
	status_t	(*close_session)(struct bone_ppp_session *session);
	
	/*
	 * Deliver data to a PPP session - data is demultiplexed to handler of 'ppp_proto'
	 */
	status_t	(*receive_data)(struct bone_ppp_session *session, struct bone_data *data, uint16 ppp_proto);
	
	/*
	 * Register/unregister handler with session to receive events and data
	 */
	status_t	(*register_handler)(struct bone_ppp_session *session, struct bone_ppp_handler *handler);
	status_t	(*unregister_handler)(struct bone_ppp_session *session, struct bone_ppp_handler *handler);
	
	/*
	* Called by handler
	*/
	void		(*handler_started)(struct bone_ppp_session *session, struct bone_ppp_handler *handler);
	void		(*handler_finished)(struct bone_ppp_session *session, struct bone_ppp_handler *handler);
	void		(*handler_up)(struct bone_ppp_session *session, struct bone_ppp_handler *handler);
	void		(*handler_down)(struct bone_ppp_session *session, struct bone_ppp_handler *handler);
	
	/*
	* Called by transport layer
	*/
	void		(*transport_layer_up)(struct bone_ppp_session *session);
	void		(*transport_layer_down)(struct bone_ppp_session *session);
	
} bone_ppp_session_info_t;

#define BONE_PPP_MODULE "network/ppp/bone_ppp"
#define BONE_PPP_MODULE_NAME "session"

#endif	/* H_BONE_PPP */
